from unicodeconverter import convertToUnicode

def evaluateBoolean( b ):
    
    if isinstance(b, bool):
        return b
    
    if isinstance(b, str):
        b = convertToUnicode(b)
    
    if isinstance(b, unicode):
        if b.lower() == u"false":
            return False
        elif b.lower() == u"true":
            return True
        elif b.lower() == u"no":
            return False
        elif b.lower() == u"yes":
            return True
        else:
            try:
                return bool( int(b) )
            except:
                return True
    else:
        try:
            return bool( int(b) )
        except:
            return True
            


import base64
import hmac
import hashlib
import urllib
import xml.etree.cElementTree as ET
from datetime import datetime
import time
import dateutil.parser
import logging
from ..pagegetter import RequestQueuer
from .lib import etree_to_dict, safe_quote_tuple


logger = logging.getLogger("main")


SDB_NAMESPACE = "{http://sdb.amazonaws.com/doc/2009-04-15/}"


def sdb_now(offset=0):
    """Return an 11 character, zero padded string with the current Unixtime.
    
    **Keyword arguments:**
     * *offset* -- Offset in seconds. (Default 0)
    """
    return str(int(offset + time.time())).zfill(11)


def sdb_now_add(seconds, offset=0):
    """Return an 11 character, zero padded string with the current Unixtime
    plus an integer.
   
    **Arguments:**
     * *seconds* -- Seconds to add to the current time.
   
    **Keyword arguments:**
     * *offset* -- Offset in seconds. (Default 0)
    """
    return str(int(offset + time.time() + seconds)).zfill(11)


def sdb_parse_time(date_string, offset=0):
    """Parse a date string, then return an 11 character, zero padded
    string with the current Unixtime plus an integer.
   
    **Arguments:**
     * *date_string* -- Date string
   
    **Keyword arguments:**
     * *offset* -- Offset in seconds. (Default 0)
    """
    parsed_time = time.mktime(dateutil.parser.parse(date_string).timetuple())
    return str(int(offset + parsed_time)).zfill(11)


def sdb_latitude(latitude):
    """Return an 8 character, zero padded string version of the
    latitude parameter.
   
    **Arguments:**
     * *latitude* -- Latitude.
    """
    adjusted = (90 + float(latitude)) * 100000
    return str(int(adjusted)).zfill(8)


def sdb_longitude(longitude):
    """Return an 8 character, zero padded string version of the
    longitude parameter.
   
    **Arguments:**
     * *longitude* -- Longitude.
    """
    adjusted = (180 + float(longitude)) * 100000
    return str(int(adjusted)).zfill(8)

class AmazonSDB:
   
    """
    Amazon Simple Database API.
    """
   
    host = "sdb.amazonaws.com"
    box_usage = 0.0
   
    def __init__(self, aws_access_key_id, aws_secret_access_key, rq=None):
        """
        **Arguments:**
         * *aws_access_key_id* -- Amazon AWS access key ID
         * *aws_secret_access_key* -- Amazon AWS secret access key
       
        **Keyword arguments:**
         * *rq* -- Optional RequestQueuer object.
        """
        if rq is None:
            self.rq = RequestQueuer()
        else:
            self.rq = rq
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.rq.setHostMaxRequestsPerSecond(self.host, 0)
        self.rq.setHostMaxSimultaneousRequests(self.host, 0)

    def createDomain(self, domain):
        """
        Create a SimpleDB domain.
       
        **Arguments:**
         * *domain* -- Domain name
        """
        parameters = {
            "Action":"CreateDomain",
            "DomainName":domain
        }
        d = self._request(parameters)
        d.addCallback(self._createDomainCallback, domain)
        return d

    def _createDomainCallback(self, data, domain):
        xml = ET.fromstring(data["response"])
        box_usage = float(xml.find(".//%sBoxUsage" % SDB_NAMESPACE).text)
        self.box_usage += box_usage
        logger.debug("Created SimpleDB domain '%s'. Box usage: %s" % (domain,
            box_usage))
        return True
   
    def deleteDomain(self, domain):
        """
        Delete a SimpleDB domain.
       
        **Arguments:**
         * *domain* -- Domain name
        """
        parameters = {}
        parameters["Action"] = "DeleteDomain"
        parameters["DomainName"] = domain
        d = self._request(parameters)
        d.addCallback(self._deleteDomainCallback, domain)
        return d

    def _deleteDomainCallback(self, data, domain):
        xml = ET.fromstring(data["response"])
        box_usage = float(xml.find(".//%sBoxUsage" % SDB_NAMESPACE).text)
        self.box_usage += box_usage
        logger.debug("Deleted SimpleDB domain '%s'. Box usage: %s" % (domain, 
            box_usage))
        return True   

    def listDomains(self):
        """
        List SimpleDB domains associated with an account.
        """
        return self._listDomains()
   
    def _listDomains(self, 
                     next_token=None, 
                     previous_results=None, 
                     total_box_usage=0):
        parameters = {}
        parameters["Action"] = "ListDomains"
        if next_token is not None:
            parameters["NextToken"] = next_token
        d = self._request(parameters)
        d.addCallback(self._listDomainsCallback, 
                      previous_results=previous_results,
                      total_box_usage=total_box_usage)
        return d

    def _listDomainsCallback(self, 
                             data, 
                             previous_results=None, 
                             total_box_usage=0):
        xml = ET.fromstring(data["response"])
        box_usage = float(xml.find(".//%sBoxUsage" % SDB_NAMESPACE).text)
        self.box_usage += box_usage
        total_box_usage += box_usage
        xml_response = etree_to_dict(xml, namespace=SDB_NAMESPACE)
        if "DomainName" in xml_response["ListDomainsResult"][0]:
            results = xml_response["ListDomainsResult"][0]["DomainName"]
        else:
            results = []
        if previous_results is not None:
            results.extend(previous_results)
        if "NextToken" in xml_response["ListDomainsResult"]:
            next_token = xml_response["ListDomainsResult"][0]["NextToken"][0]
            return self._listDomains(next_token=next_token,
                                     previous_results=results,
                                     total_box_usage=total_box_usage)
        logger.debug("Listed domains. Box usage: %s" % total_box_usage)
        return results

    def domainMetadata(self, domain):
        """
        Return meta-information about a domain.
       
        **Arguments:**
         * *domain* -- Domain name
        """
        parameters = {}
        parameters["Action"] = "DomainMetadata"
        parameters["DomainName"] = domain
        d = self._request(parameters)
        d.addCallback(self._domainMetadataCallback, domain)
        return d
       
    def _domainMetadataCallback(self, data, domain):
        xml = ET.fromstring(data["response"])
        box_usage = float(xml.find(".//%sBoxUsage" % SDB_NAMESPACE).text)
        self.box_usage += box_usage
        logger.debug("Got SimpleDB domain '%s' metadata. Box usage: %s" % (
            domain,
            box_usage))
        xml_response = etree_to_dict(xml, namespace=SDB_NAMESPACE)
        return xml_response["DomainMetadataResult"][0]
   
    def putAttributes(self, domain, item_name, attributes, replace=None):
        """
        Put attributes into domain at item_name.
       
        **Arguments:**
         * *domain* -- Domain name
         * *item_name* -- Item name
         * *attributes* -- Dictionary of attributes
       
        **Keyword arguments:**
         * *replace* -- List of attributes that should be overwritten
           (Default empty list)
        """
        if replace is None:
            replace = []
        if not isinstance(replace, list):
            raise Exception("Replace argument must be a list.")
        if not isinstance(attributes, dict):
            raise Exception("Attributes argument must be a dictionary.")
        parameters = {}
        parameters["Action"] = "PutAttributes"
        parameters["DomainName"] = domain
        parameters["ItemName"] = item_name
        attributes_list = []
        for attribute in attributes.items():
            # If the attribute is a list, split into multiple attributes.
            if isinstance(attribute[1], list):
                for value in attribute[1]:
                    attributes_list.append((attribute[0], value))
            else:
                attributes_list.append(attribute)
        i = 0
        for attribute in attributes_list:
            parameters["Attribute.%s.Name" % i] = attribute[0]
            parameters["Attribute.%s.Value" % i] = attribute[1]
            if attribute[0] in replace:
                parameters["Attribute.%s.Replace" % i] = "true"
            i += 1
        d = self._request(parameters)
        d.addCallback(self._putAttributesCallback, domain, item_name, attributes)
        return d
       
    def _putAttributesCallback(self, data, domain, item_name, attributes):
        xml = ET.fromstring(data["response"])
        box_usage = float(xml.find(".//%sBoxUsage" % SDB_NAMESPACE).text)
        self.box_usage += box_usage
        logger.debug("""Put attributes %s on '%s' in SimpleDB domain '%s'. Box usage: %s""" % (
            attributes,
            item_name,
            domain,
            box_usage))
        return True
   
    def getAttributes(self, domain, item_name, attribute_name=None):
        """
        Get one or all attributes from domain at item_name.
       
        **Arguments:**
         * *domain* -- Domain name
         * *item_name* -- Item name
       
        **Keyword arguments:**
         * *attribute_name* -- Name of specific attribute to get (Default None)
        """
        parameters = {}
        parameters["Action"] = "GetAttributes"
        parameters["DomainName"] = domain
        parameters["ItemName"] = item_name
        if attribute_name is not None:
            parameters["AttributeName"] = attribute_name
        d = self._request(parameters)
        d.addCallback(self._getAttributesCallback, domain, item_name)
        return d
       
    def _getAttributesCallback(self, data, domain, item_name):
        xml = ET.fromstring(data["response"])
        box_usage = float(xml.find(".//%sBoxUsage" % SDB_NAMESPACE).text)
        self.box_usage += box_usage
        logger.debug("""Got attributes from '%s' in SimpleDB domain '%s'. Box usage: %s""" % (
            item_name,
            domain,
            box_usage))
        xml_response = etree_to_dict(xml, namespace=SDB_NAMESPACE)
        attributes = {}
        if xml_response["GetAttributesResult"][0] is None:
            raise Exception("Item does not exist.")
        for attribute in xml_response["GetAttributesResult"][0]['Attribute']:
            if attribute["Name"][0] not in attributes:
                attributes[attribute["Name"][0]] = []
            attributes[attribute["Name"][0]].extend(attribute["Value"])
        return attributes

    def delete(self, domain, item_name):
        """
        Delete all attributes from domain at item_name.
       
        **Arguments:**
         * *domain* -- Domain name
         * *item_name* -- Item name
        """
        return self.deleteAttributes(domain, item_name)

    def deleteAttributes(self, domain, item_name, attributes=None):
        """
        Delete one or all attributes from domain at item_name.
       
        **Arguments:**
         * *domain* -- Domain name
         * *item_name* -- Item name
       
        **Keyword arguments:**
         * *attributes* -- List of attribute names, or dictionary of
           attribute name / value pairs. (Default empty dict)
        """       
        if attributes is None:
            attributes = {}
        if not isinstance(attributes, dict) and \
           not isinstance(attributes, list):
            message = "Attributes parameter must be a dictionary or a list."
            raise Exception(message)
        parameters = {}
        parameters["Action"] = "DeleteAttributes"
        parameters["DomainName"] = domain
        parameters["ItemName"] = item_name
        if isinstance(attributes, dict):
            attr_count = 1
            for key in attributes:
                parameters["Attribute.%s.Name" % attr_count] = key
                parameters["Attribute.%s.Value" % attr_count] = attributes[key]
                attr_count += 1
        if isinstance(attributes, list):
            attr_count = 0
            for key in attributes:
                parameters["Attribute.%s.Name" % attr_count] = key
                attr_count += 1
        d = self._request(parameters)
        d.addCallback(self._deleteAttributesCallback, domain, item_name)
        return d

    def _deleteAttributesCallback(self, data, domain, item_name):
        xml = ET.fromstring(data["response"])
        box_usage = float(xml.find(".//%sBoxUsage" % SDB_NAMESPACE).text)
        self.box_usage += box_usage
        logger.debug("""Deleted attributes from '%s' in SimpleDB domain '%s'. Box usage: %s""" % (
            item_name,
            domain,
            box_usage))
        return True
   
    def select(self, select_expression):
        """
        Run a select query
       
        **Arguments:**
         * *select_expression* -- Select expression
        """   
        return self._select(select_expression)
       
    def _select(self, select_expression, next_token=None, 
                previous_results=None,
                total_box_usage=0):
        parameters = {}
        parameters["Action"] = "Select"
        parameters["SelectExpression"] = select_expression
        if next_token is not None:
            parameters["NextToken"] = next_token
        d = self._request(parameters)
        d.addCallback(self._selectCallback, 
                      select_expression=select_expression,
                      previous_results=previous_results,
                      total_box_usage=total_box_usage)
        return d

    def _selectCallback(self, data, select_expression=None, 
                        previous_results=None,
                        total_box_usage=0):
        if previous_results is not None:
            results = previous_results
        else:
            results = {}
        xml = ET.fromstring(data["response"])
        box_usage = float(xml.find(".//%sBoxUsage" % SDB_NAMESPACE).text)
        self.box_usage += box_usage
        total_box_usage += box_usage
        next_token_element = xml.find(".//%sNextToken" % SDB_NAMESPACE)
        if next_token_element is not None:
            next_token = next_token_element.text
        else:
            next_token = None
        items = xml.findall(".//%sItem" % SDB_NAMESPACE)
        results = {}
        for item in items:
            key = item.find("./%sName" % SDB_NAMESPACE).text
            attributes = item.findall("%sAttribute" % SDB_NAMESPACE)
            attribute_dict = {}
            for attribute in attributes:
                attr_name = attribute.find("./%sName" % SDB_NAMESPACE).text
                attr_value = attribute.find("./%sValue" % SDB_NAMESPACE).text
                if attr_name in attribute_dict:  
                    attribute_dict[attr_name].append(attr_value)
                else:
                    attribute_dict[attr_name] = [attr_value]
            results[key] = attribute_dict
        if next_token is not None:
            return self._select(select_expression, next_token=next_token,
                                previous_results=results,
                                total_box_usage=total_box_usage)
        logger.debug("""Select:\n'%s'\nBox usage: %s""" % (
            select_expression,
            total_box_usage))
        return results       

    def _request(self, parameters):
        """
        Add authentication parameters and make request to Amazon.
       
        **Arguments:**
         * *parameters* -- Key value pairs of parameters
        """
        parameters = self._getAuthorization("GET", parameters)
        query_string = urllib.urlencode(parameters)
        url = "https://%s/?%s" % (self.host, query_string)
        d = self.rq.getPage(url, method="GET")
        return d
         
    def _canonicalize(self, parameters):
        """
        Canonicalize parameters for use with AWS Authorization.
       
        **Arguments:**
         * *parameters* -- Key value pairs of parameters
       
        **Returns:**
         * A safe-quoted string representation of the parameters.
        """
        parameters = parameters.items()
        parameters.sort(lambda x, y:cmp(x[0], y[0]))
        return "&".join([safe_quote_tuple(x) for x in parameters])
       
    def _getAuthorization(self, method, parameters):
        """
        Create authentication parameters.
       
        **Arguments:**
         * *method* -- HTTP method of the request
         * *parameters* -- Key value pairs of parameters
       
        **Returns:**
         * A dictionary of authorization parameters
        """
        signature_parameters = {
            "AWSAccessKeyId":self.aws_access_key_id,
            "SignatureVersion":"2",
            "SignatureMethod":"HmacSHA256",
            'Timestamp':datetime.utcnow().isoformat()[0:19]+"+00:00",
            "AWSAccessKeyId":self.aws_access_key_id,
            "Version":"2009-04-15"
        }
        signature_parameters.update(parameters)
        query_string = self._canonicalize(signature_parameters)
        string_to_sign = "%(method)s\n%(host)s\n%(resource)s\n%(qs)s" % {
            "method":method,
            "host":self.host.lower(),
            "resource":"/",
            "qs":query_string,
        }
        args = [self.aws_secret_access_key, string_to_sign, hashlib.sha256]
        signature = base64.encodestring(hmac.new(*args).digest()).strip()
        signature_parameters.update({'Signature': signature})
        return signature_parameters



from twisted.web.resource import Resource
from twisted.web import server
import traceback
import simplejson
from twisted.python.failure import Failure
from twisted.internet import reactor
from awspider import evaluateboolean
import urlparse

class PeerResource(Resource):
    
    isLeaf = True
    
    def __init__( self, spider ):
        
        self.spider = spider
        Resource.__init__(self)
    
    def render(self, request):
        
        request.setHeader('Content-type', 'text/javascript; charset=UTF-8')
        
        if len(request.postpath) > 0:
            if request.postpath[0] == "check":
                reactor.callLater( 5, self.spider.checkPeers )
                return simplejson.dumps( True )  
            if request.postpath[0] == "getpage" and "url" in request.args:
                #print "Recieving peering request for %s" % request.args["url"][0]
                kwargs = {}
                if "method" in request.args:
                    kwargs["method"] = request.args["method"][0]     
                if "postdata" in request.args: 
                    kwargs["postdata"] = urlparse.parse_qs( request.args["postdata"][0] )
                if "headers" in request.args: 
                    kwargs["headers"] = urlparse.parse_qs( request.args["headers"][0] )  
                if "cookies" in request.args: 
                    kwargs["cookies"] = urlparse.parse_qs( request.args["cookies"][0] )           
                if "agent" in request.args:
                    kwargs["agent"] = request.args["agent"][0]
                if "timeout" in request.args:
                    kwargs["timeout"] = int( request.args["timeout"][0] )
                if "followRedirect" in request.args:
                    kwargs["followRedirect"] = evaluateboolean( request.args["followRedirect"][0] )
                if "hash_url" in request.args: 
                    kwargs["hash_url"] = request.args["hash_url"][0] 
                if "cache" in request.args: 
                    kwargs["cache"] = int( request.args["cache"][0] )
                if "prioritize" in request.args: 
                    kwargs["prioritize"] = evaluateboolean( request.args["prioritize"][0] )
    
                d = self.spider.pg.getPage( request.args["url"][0], **kwargs  )              
                d.addCallback( self._getpageCallback, request )         
                d.addErrback( self._getpageErrback, request )     
                return server.NOT_DONE_YET
    
    def _getpageCallback( self, data, request ):

        request.setResponseCode( data["status"], data["message"] )

        for header in data["headers"]:
            request.setHeader(header, data["headers"][header][0])
        request.write( data["response"] )
        request.finish()
        
    def _getpageErrback( self, error, request ):
        reason = str(error.value)
        tb = traceback.format_exc( traceback.extract_tb(error.tb) )
        request.write( simplejson.dumps( {"error":reason, "traceback":tb} ) )    
        request.finish()
        
    def _successResponse( self, data ):
        return simplejson.dumps( True )

    def _errorResponse( self, error ):

        reason = str(error.value)
        tb = traceback.format_exc( traceback.extract_tb(error.tb) )

        return simplejson.dumps( {"error":reason, "traceback":tb} )

    def _immediateResponse(self, data, request ):           
        request.write( data )
        request.finish()


from twisted.internet.defer import DeferredList
from twisted.web.client import HTTPClientFactory, _parse
from twisted.internet import reactor

import xml.etree.cElementTree as ET

import logging
logger = logging.getLogger("main")

import random
import socket
            
def getPage( url, method='GET', postdata=None, headers=None, agent="AWSpider", timeout=60, cookies=None, followRedirect=1 ):
    
    scheme, host, port, path = _parse(url)

    factory = HTTPClientFactory(
        url, 
        method=method, 
        postdata=postdata,
        headers=headers, 
        agent=agent, 
        timeout=timeout, 
        cookies=cookies,
        followRedirect=followRedirect
    )

    if scheme == 'https':
        from twisted.internet import ssl
        contextFactory = ssl.ClientContextFactory()
        reactor.connectSSL(host, port, factory, contextFactory, timeout=timeout)
    else:
        reactor.connectTCP(host, port, factory, timeout=timeout)

    return factory.deferred

class NetworkAddressGetter():
    
    local_ip = None
    public_ip = None
    
    def __init__( self ):
        self.ip_functions = [self.getDomaintools, self.getShowmyip, self.getIPPages, self.getIPLocationtools]
        random.shuffle(self.ip_functions)
    
    def __call__( self ):
        d = self.getAmazonIPs()
        d.addCallback( self._getAmazonIPsCallback )
        d.addErrback( self.getPublicIP )
        return d
    
    def getAmazonIPs( self ):
        logger.debug( "Getting local IP from Amazon." )
        a = getPage( "http://169.254.169.254/2009-04-04/meta-data/local-ipv4", timeout=5 )

        logger.debug( "Getting public IP from Amazon." )
        b = getPage( "http://169.254.169.254/2009-04-04/meta-data/public-ipv4", timeout=5  )
        
        d = DeferredList([a,b], consumeErrors=True)
        return d
    
    def _getAmazonIPsCallback( self, data ):
        
        if data[0][0] == True:
            self.local_ip = data[0][1]
            logger.debug( "Got local IP %s from Amazon." % self.local_ip )
        else:
            logger.debug( "Could not get local IP from Amazon." )

        if data[1][0] == True:
            public_ip = data[1][1]
            logger.debug( "Got public IP %s from Amazon." % public_ip )
            
            response = {}
            if self.local_ip is not None:
                response["local_ip"] = self.local_ip
            response["public_ip"] = public_ip
            
            return response
            
        else:
            logger.debug( "Could not get public IP from Amazon." )
            raise Exception( "Could not get public IP from Amazon." )
            
        
    def getPublicIP( self, error=None ):
        
        if len(self.ip_functions) > 0:
            func = self.ip_functions.pop()
            d = func()
            d.addCallback( self._getPublicIPCallback )
            d.addErrback( self.getPublicIP )
            return d
        else:
            logger.error( "Unable to get public IP address. Check your network connection" )
            response = {}
            if self.local_ip is not None:
                response["local_ip"] = self.local_ip
            else:
                response["local_ip"] = socket.gethostbyname(socket.gethostname())
            return response
             
    def _getPublicIPCallback( self, public_ip ):
        response = {}
        response["public_ip"] = public_ip
        if self.local_ip is not None:
            response["local_ip"] = self.local_ip
        else:
            response["local_ip"] = socket.gethostbyname(socket.gethostname())
        return response
    
    def getIPLocationtools(self):
        logger.debug( "Getting public IP from iplocationtools.com." )
        d = getPage( "http://iplocationtools.com/ip_query.php", timeout=5 )
        d.addCallback( self._getIPLocationtoolsCallback )
        return d       

    def _getIPLocationtoolsCallback(self, data ):
        domaintools_xml = ET.XML( data )
        public_ip = domaintools_xml.find("Ip").text
        logger.debug( "Got public IP %s from iplocationtools.com." % public_ip ) 
        return public_ip

    def getIPPages(self):
        logger.debug( "Getting public IP from ippages.com." )
        d = getPage( "http://www.ippages.com/xml/", timeout=5 )
        d.addCallback( self._getIPPagesCallback )
        return d       
    
    def _getIPPagesCallback(self, data ):
        domaintools_xml = ET.XML( data )
        public_ip = domaintools_xml.find("ip").text
        logger.debug( "Got public IP %s from ippages.com." % public_ip ) 
        return public_ip
    
    def getDomaintools(self):
        logger.debug( "Getting public IP from domaintools.com." )
        d = getPage( "http://ip-address.domaintools.com/myip.xml", timeout=5 )
        d.addCallback( self._getDomaintoolsCallback )
        return d
        
    def _getDomaintoolsCallback(self, data):
        domaintools_xml = ET.XML( data )
        public_ip = domaintools_xml.find("ip_address").text
        logger.debug( "Got public IP %s from domaintools.com." % public_ip ) 
        return public_ip
            
    def getShowmyip(self):     
        logger.debug( "Getting public IP from showmyip.com." )
        d = getPage( "http://www.showmyip.com/xml/", timeout=5  )
        d.addCallback( self._getShowmyipCallback )
        return d
        
    def _getShowmyipCallback(self, data ):
        showmyip_xml = ET.XML( data )
        public_ip = showmyip_xml.find("ip").text
        logger.debug( "Got public IP %s from showmyip.com." % public_ip )
        return public_ip

def getNetworkAddress():
    n = NetworkAddressGetter()
    d = n()
    d.addCallback( _getNetworkAddressCallback )
    return d
    
def _getNetworkAddressCallback( data ):
    return data    
    

if __name__ == "__main__":

    import logging.handlers
    handler = logging.StreamHandler()
    formatter = logging.Formatter("%(levelname)s: %(message)s %(pathname)s:%(lineno)d")
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.setLevel(logging.DEBUG)
    reactor.callWhenRunning( getNetworkAddress )     
    reactor.run()


from twisted.web.resource import Resource
from genshi.template import TemplateLoader
import os
import cStringIO, gzip

class TemplateResource(Resource):

    isLeaf = True

    def __init__(self, path = None):
        self.path = path

    loader = TemplateLoader( search_path=[ os.path.join(os.path.dirname(__file__), '../web_templates') ] , auto_reload=True )

    def render_GET(self, request):
        if self.path is not None:
            content = self._render_template( self.path.replace("docs/", "") + ".genshi" )
        else:
            content = self._render_template( request.path.replace("docs/", "").strip("/") + ".genshi" )
        
        content = content.replace("\t", "")
        encoding = request.getHeader("accept-encoding")

        if encoding and "gzip" in encoding:

            zbuf = cStringIO.StringIO()
            zfile = gzip.GzipFile(None, 'wb', 9, zbuf)
            if isinstance( content, unicode ):
                zfile.write( unicode(content).encode("utf-8") )
            elif isinstance( content, str ):
                zfile.write( unicode(content, 'utf-8' ).encode("utf-8") )
            else:
                zfile.write( unicode(content).encode("utf-8") )


            zfile.close()
            request.setHeader("Content-encoding","gzip")
            return zbuf.getvalue()
        else:
            return content
        
        
    def _render_template(self, template, data=None):
        if data is None:
            data = {}
        t = self.loader.load( template )
        return t.generate( data=data ).render('xhtml', doctype='xhtml')

